/*
This program creates the following pipelines using a Java process
as the parent (it doesn't matter if the children processes are written
in Java or C or any other language).
                                                    parent
                                                +------------+
                                                |            |
stdin--+---------------------------------------->>0        1>>--->stdout
       |                                        |            |
       |      child_1                 pipe      |            |
       |   +------------+   +------0========0--->>3        2>>--->stderr
       |   |            |   |                   |            |
       +--->>0        1>>---+   +--0========0--->>4          |
       |   |            |       |     pipe      |            |
       |   |          2>>       |               +------------+
       |   |            |       |           CreateTwoPipedChildren_Threaded.class
       |   +------------+       |
       |   weird_echo.exe       |
       |                        |
       |                        |
       |      child_2           |
       |   +------------+       |
       |   |            |       |
       +--->>0        1>>-------+
           |            |
           |          2>>
           |            |
           +------------+
           weird_echo.exe

To set up these pipelines, we need to create each child process,
have each child inherit its stdin from the parent, and then
connect each child's stdout to a (new) input stream in the
parent (surprisingly in Java there is no need to explicitly
create "pipe" objects).

Notice how the parent's stdin ends up being connected to all three
processes. This is not ideal. In the Windows C version of this program,
each process has its own independent input stream. I cannot figure out
how to do that with Java.

This program (the parent process) reads and filters one line of text
at a time from its three input streams (denoted 0, 3, and 4) in parallel,
using three threads. The main() thread reads from stream 0. This process
creates two new threads, one to read from stream 3 and one to read from
stream 4.

EXERCISE: Modify this program to put a weird_echo.exe pipeline in the
parent's stdin stream, so the parent is reading characters that have
been filtered by an instance of weird_echo.exe (a child_0).
*/

import java.io.IOException;
import java.util.Scanner;

public class CreateTwoPipedChildren_Threaded
{
   public static void main(String[] args) throws IOException, InterruptedException
   {
      // Create a command line for running first child.
      ProcessBuilder pb1 = new ProcessBuilder("weird_echo.exe");
    //ProcessBuilder pb1 = new ProcessBuilder("cmd.exe", "/C", "start", "weird_echo.exe");
      // Have the child inherit the parent's standard input stream.
      pb1.redirectInput( ProcessBuilder.Redirect.INHERIT );
      // Execute the child.
      Process process1 = pb1.start();  // this throws IOException


      // Create a command line for running second child.
      ProcessBuilder pb2 = new ProcessBuilder("weird_echo.exe");
    //ProcessBuilder pb2 = new ProcessBuilder("cmd.exe", "/C", "start", "weird_echo.exe");
      // Have the child inherit the parent's standard input stream.
      pb2.redirectInput( ProcessBuilder.Redirect.INHERIT );
      // Execute the child.
      Process process2 = pb2.start();  // this throws IOException


      // Get an input stream that reads from the first child's stdout.
      // This is the first "pipeline" from the first child into the parent.
      Scanner scanner1 = new Scanner( process1.getInputStream() );

      // Get an input stream hato reads from the second child's stdout.
      // This is the second "pipeline" from the second child into the parent.
      Scanner scanner2 = new Scanner( process2.getInputStream() );

      // Create a Scanner object to make it easier to use System.in.
      Scanner scanner0 = new Scanner( System.in );


      FileThread thread1 = new FileThread(scanner1, 2); // double vowels
      FileThread thread2 = new FileThread(scanner2, 3); // triple vowels
      FileThread thread0 = new FileThread(scanner0, 4); // quadruple vowels

      thread1.start(); // asynchronous thread
      thread2.start(); // asynchronous thread
      thread0.run();   // synchronous function call

      // Wait for each child to finish its work.
      try
      {
         thread1.join();
         thread2.join();
      }
      catch (InterruptedException e){e.getMessage();}
   }
}


// A FileThread will read every line of input
// from the Scanner passed to its constructor.
class FileThread extends Thread
{
   Scanner scanner;
   int count;

   public FileThread(Scanner scanner, int count)
   {
      this.scanner = scanner;
      this.count = count;
   }

   public void run()
   {
      // Read and filter each line of data from the scanner.
      while ( scanner.hasNextLine() )
      {
         String oneLine = scanner.nextLine();

         // Filter the line of input while printing it.
         for (int i = 0; i < oneLine.length(); i++)
         {
            System.out.print( oneLine.charAt(i) );
            if ( -1 != "aeiouAEIOU".indexOf( oneLine.charAt(i) ) )
            {
               for (int j = 0; j < count - 1; j++)
               {
                  System.out.print( oneLine.charAt(i) ); // duplicate vowels
               }
            }
         }
         System.out.println();
      }
      if (4 == count)
         System.out.println("Parent ended.");
      else
         System.out.println("Child " + (count-1) + " ended.");
   }
}//FileThread
