/*
   search for "aardvark"
*/

import java.io.File;
import java.io.FileInputStream;
import java.io.BufferedInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Scanner;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.text.SimpleDateFormat;
import java.util.Date;

public class Pipelined
{
   public static final SimpleDateFormat ft = new SimpleDateFormat("hh:mm:ss");

   public static void main(String[] args) throws FileNotFoundException, IOException
   {
      // Open the file.
      int bufferSize = 16 * 1024 * 1024;
      BufferedInputStream bis = new BufferedInputStream(
                                new FileInputStream("result"), bufferSize);
      Scanner scanner = new Scanner(bis);

      ExecutorService pool = Executors.newFixedThreadPool(12);

      System.out.println("Start Time " + ft.format(new Date()));

      int lineNumber = 0;
      while ( scanner.hasNextLine() ) // this blocks
      {
         ++lineNumber;
         pool.execute( new Task(scanner.nextLine(), lineNumber) );
      }

      System.out.println("End Time " + ft.format(new Date()));
      bis.close();
      System.out.println( pool.toString() );
      pool.shutdown();
   }
}


// A Task object, when it is called by one of the threads in
// the Executor's thread pool, will filter one line of text.
class Task implements Runnable
{
   private final String oneLine;
   private final int lineNumber;

   public Task(final String oneLine, int lineNumber)
   {
      this.oneLine = oneLine;
      this.lineNumber = lineNumber;
   }

   public void run()
   {
      // Filter the line of input.
      // https://stackoverflow.com/questions/7384791/splitting-strings-through-regular-expressions-by-punctuation-and-whitespace-etc
      String[] tokens =  oneLine.trim().split("[\\p{Punct}\\s]+");

      for (String s : tokens)
      {
            if ( s.equals("aardvark") )
               System.out.println(Pipelined.ft.format(new Date()) + " " +lineNumber);
      }
   }
}//Task
