/*
   parallel search for "aardvark" in one file called "data"
*/

import java.io.File;
import java.io.FileInputStream;
import java.io.BufferedInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Scanner;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.time.Instant;
import java.time.Duration;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

public class SearchParallel
{
   public static final SimpleDateFormat ft = new SimpleDateFormat("hh:mm:ss");

   public static void main(String[] args) throws FileNotFoundException, IOException
   {
      // Open the file.
      int bufferSize = 16 * 1024 * 1024;
      BufferedInputStream bis = new BufferedInputStream(
                                new FileInputStream("data"), bufferSize);
      Scanner scanner = new Scanner(bis);

      ExecutorService pool = Executors.newFixedThreadPool(12);

      System.out.println(ft.format(new Date()));
      Instant start = Instant.now();
      int lineNumber = 0;
      while ( scanner.hasNextLine() ) // this blocks
      {
         ++lineNumber;
         pool.execute( new Task(scanner.nextLine(), lineNumber) );
      }
      Instant stop = Instant.now();
      System.out.println(ft.format(new Date()));
      long time = Duration.between(start, stop).toMillis();
      System.out.println("elapsed time: " + time + "ms");
      bis.close();
      System.out.println( pool.toString() );
      pool.shutdown();
   }
}


// A Task object, when it is called by one of the threads in
// the Executor's thread pool, will filter one line of text.
class Task implements Runnable
{
   private final String oneLine;
   private final int lineNumber;

   public Task(final String oneLine, int lineNumber)
   {
      this.oneLine = oneLine;
      this.lineNumber = lineNumber;
   }

   public void run()
   {
      // Filter the line of input.
      String[] tokens =  oneLine.trim().split("[\\p{Punct}\\s]+");

      for (String s : tokens)
      {
         if ( s.equals("aardvark") )
            System.out.println(SearchParallel.ft.format(new Date()) + " " + lineNumber);
      }
   }
}//Task
